package cn.com.qmth.am.task;

import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.concurrent.ExecutorService;
import java.util.concurrent.LinkedBlockingQueue;
import java.util.concurrent.ThreadPoolExecutor;
import java.util.concurrent.TimeUnit;

import org.apache.commons.collections4.CollectionUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.scheduling.annotation.Scheduled;
import org.springframework.stereotype.Service;

import com.qmth.boot.core.concurrent.service.ConcurrentService;

import cn.com.qmth.am.consumer.BuildImageConsumer;
import cn.com.qmth.am.entity.QuestionEntity;
import cn.com.qmth.am.entity.StudentEntity;
import cn.com.qmth.am.enums.LockType;
import cn.com.qmth.am.service.QuestionService;
import cn.com.qmth.am.service.StudentService;
import cn.com.qmth.am.utils.SpringContextHolder;

@Service
public class BiuldImageJob {
	@Autowired
	private StudentService studentService;
	@Autowired
	private QuestionService questionService;
	@Autowired
	private ConcurrentService concurrentService;

	private static ExecutorService executor;

	static {
		int threadCount = Runtime.getRuntime().availableProcessors();
		executor = new ThreadPoolExecutor(threadCount, threadCount, 0L, TimeUnit.SECONDS,
				new LinkedBlockingQueue<>(threadCount * 2), r -> {
					Thread t = new Thread(r);
					return t;
				}, (r, executor) -> {
					if (!executor.isShutdown()) {
						try {
							executor.getQueue().put(r);
						} catch (InterruptedException e) {
							throw new RuntimeException(e);
						}
					}
				});

	}

	@Scheduled(fixedDelay = 30 * 1000, initialDelay = 20 * 1000)
	public void doJob() {
		boolean lock = concurrentService.getReadWriteLock(LockType.OCR.name()).writeLock().tryLock();
		try {
			if (!lock) {
				return;
			}
			this.dispose();
		} finally {
			if (lock) {
				concurrentService.getReadWriteLock(LockType.OCR.name()).writeLock().unlock();
			}
		}
	}

	private void dispose() {
		List<StudentEntity> stus = studentService.findToDispose();
		if (CollectionUtils.isEmpty(stus)) {
			return;
		}
		List<QuestionEntity> qs=questionService.list();
		if (CollectionUtils.isEmpty(qs)) {
			return;
		}
		Map<String,Map<Long,QuestionEntity>> qmap=new HashMap<>();
		for(QuestionEntity q:qs) {
			String key=q.getExamId()+"-"+q.getSubjectCode();
			Map<Long,QuestionEntity> tem=qmap.get(key);
			if(tem==null) {
				tem=new HashMap<>();
				qmap.put(key, tem);
			}
			tem.put(q.getId(),q);
		}
		for (StudentEntity stu : stus) {
			String key=stu.getExamId()+"-"+stu.getSubjectCode();
			Map<Long,QuestionEntity> tem=qmap.get(key);
			if(tem!=null) {
				BuildImageConsumer com = SpringContextHolder.getBean(BuildImageConsumer.class);
				com.setStudent(stu);
				com.setQuetions(tem);
				executor.execute(com);
			}
		}
	}

}
